local DQ = LibStub("AceAddon-3.0"):NewAddon("DMFQuest", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DMFQuest")
local B = LibStub("LibBabble-SubZone-3.0")
local BL = B:GetLookupTable()

--Random init stuff
local ver = GetAddOnMetadata("DMFQuest", "Version")
local fontName, fontHeight, fontFlags = GameFontNormal:GetFont()
local height = math.floor(fontHeight+0.5)
local pinWin = false
local isDMF = false
local hx = 0.36846119165421 -- 1.3.0
local hy = 0.35865038633347 -- 1.3.0
local t = 0 -- 1.3.0
local delay = 5 -- 1.3.0
local boot = 5 -- 1.3.0

--Quests and Items in pairs
local taul = {
	[29443] = 71635, --Imbued Crystal
	[29464] = 71716, --Soothsayer's Runes
	[29446] = 71638, --Ornate Weapon
	[29451] = 71715, --A Treatise on Strategy
	[29444] = 71636, --Monstrous Egg
	[29445] = 71637, --Mysterious Grimoire
	[29457] = 71952, --Captured Insignia
	[29456] = 71951, --Banner of the Fallen
	[29458] = 71953, --Fallen Adventurer's Journal
}

for _, itemID in pairs(taul) do -- 1.2.3 Trying to pre-cache quest items
	_ = GetItemInfo(itemID) -- 1.3.1
end

local palchemy = GetSpellInfo(2259)
local pinscription = GetSpellInfo(45357)
local pleatherworking = GetSpellInfo(2108)
local ptailoring = GetSpellInfo(3908)
local parchaeology = GetSpellInfo(78670)
local pcooking = GetSpellInfo(2550) -- 1.3.0
local pfirstaid = GetSpellInfo(3273) -- 1.3.0
local pfishing = GetSpellInfo(7620) -- 1.3.0

local mjname = GetItemInfo(1645)
local lpname = GetItemInfo(39354)
local sbname = GetItemInfo(6529)
local ctname = GetItemInfo(2320)
local bdname = GetItemInfo(6260)
local rdname = GetItemInfo(2604)
local sfname = GetItemInfo(30817)
local fafname = GetCurrencyInfo(393)

local function tEnter(title,text) --Show Tooltip
	GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
	GameTooltip:SetText(title)
	if text then
		GameTooltip:AddLine(text, 1, 1, 1)
	end
	GameTooltip:Show()
end

local function tLeave() --Hide Tooltip
	GameTooltip:Hide()
end

local function getSlot(id) --Returns location of an item in your backbags
	local x, y = 0
	for bag = 0, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local itemID = GetContainerItemID(bag, slot)
			if itemID == id then
				x = bag
				y = slot
				break
			end
		end
	end
	return x, y
end

local function fQuest(id) --Is Player on Quest?
	local foundIt = false
	local questsNum = select(-1, GetNumQuestLogEntries())
	for y = 1, questsNum do
		local questID = select(-3, GetQuestLogTitle(y))
		if questID == id then
			foundIt = true
			break
		end
	end
	return foundIt
end

local function skinIcon(frame) --Skin Icons to fit ElvUI or TukUI
	if IsAddOnLoaded("ElvUI") then
		local E, L, V, P, G, DF = unpack(ElvUI)
		frame.texture:SetTexCoord(unpack(E.TexCoords))
		frame.texture:SetInside()
	else
		frame:SetTexCoord(.07, .93, .07, .93)
		frame:Point("TOPLEFT", 2, -2)
		frame:Point("BOTTOMRIGHT", -2, 2)
	end
end

local function loopDeLoop(self, elapsed) -- Update timer for Horde portalzone detector, -- 1.3.0
	t = t + elapsed
	if (t >= delay) then
		--DQ:Print("Looped: "..t) -- Debug
		DQ:CheckDMPZ()
		t = 0
	end
end


local function distance(x, y, px, py) -- Calculate the distance between point A and point B, -- 1.3.0
	local dist = sqrt((x*100-px*100)^2+(y*100-py*100)^2) or 0
	dist = math.floor(dist+0.5)
	return dist
end

--Text-/Main-frame
local Output = CreateFrame("Frame", "DMFQOutput", UIParent)
Output:SetMovable(true)
Output:SetClampedToScreen(true)
Output:EnableMouse(true)
Output:RegisterForDrag("LeftButton")
Output:SetScript("OnDragStart", Output.StartMoving)
Output:SetScript("OnDragStop", Output.StopMovingOrSizing)
Output:SetBackdrop(StaticPopup1:GetBackdrop())

Output:SetHeight(120)
Output:SetWidth(225)

Output.text = Output:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
Output.text:SetAllPoints()
Output.text:SetText("- DMFQuest -")
Output:SetPoint("CENTER", 0, 100)

--Quest Item Buttons
for i = 1, 9 do -- 1.3.2
	Output.Buttons = Output.Buttons or {}
	Output.Buttons[i] = CreateFrame("Button", "DMFQItemTab"..i, Output)
end

for _,ibutton in ipairs(Output.Buttons) do -- 1.3.2
	ibutton:SetSize(25,25)
	ibutton:SetScript("OnLeave",function() tLeave() end)
	ibutton:ClearAllPoints()
end

-- Keep track and update players location in Mulgore for Horde players
Output:SetScript("OnUpdate", loopDeLoop) -- 1.3.2

function DQ:CheckDMF() --Is Darkmoon Faire avaible?
	local _, _, day, _ = CalendarGetDate(); -- 1.3.0
--	local day = 15 -- Debug

	for y = 1, 3 do
		local _, _, hIcon = CalendarGetHolidayInfo(0, day, y) -- 1.3.0
		if hIcon == "calendar_darkmoonfaireterokkar" then
			isDMF = true
		end
	end
end

--Slash Commands
function DQ:SlashCMD(arg)
	local arg = arg:trim() -- 1.3.0 Turned into local
	if not arg or arg == "" then
		self:Print(L["snoarg"]);
	elseif arg == "enable" or arg == "toggle" or arg == "true" or arg == "show" then
		pinWin = true
		self:CheckDMF()
		self:LineCheck()
		self:RegisterEvent("BAG_UPDATE", "LineCheck")
		self:RegisterEvent("QUEST_ACCEPTED", "LineCheck")
		Output:Show()
		self:Print(L["sshow"])
	elseif arg == "disable" or arg == "false" or arg == "hide" then
		pinWin = false
		self:UnregisterEvent("BAG_UPDATE")
		self:UnregisterEvent("QUEST_ACCEPTED", "LineCheck")
		Output:Hide()
		self:Print(L["shide"])
	elseif arg == "hd" then -- Debug for 1.3.0
		SetMapToCurrentZone()
		local px, py = GetPlayerMapPosition("player")
		local dist = distance(hx, hy, px, py)
		self:Print("x: "..px..", y: "..py..", d: "..dist)		
	else
		self:Print(L["sunknown"]);
	end
end

--Is Player in Darkmoon Faire Portal Zone?
function DQ:CheckDMPZ()
	SetMapToCurrentZone()
	local px, py = GetPlayerMapPosition("player") -- 1.3.0
	if (GetRealZoneText() == BL["Mulgore"] and GetSubZoneText() == "") then -- 1.3.0
		if isDMF then
			Output:SetScript("OnUpdate", loopDeLoop)
		elseif boot > 0 then -- Loop 5 times if just logged in to check if DMF is avaible
			self:CheckDMF()
			Output:SetScript("OnUpdate", loopDeLoop)
			boot = boot - 1
		else
			Output:SetScript("OnUpdate", nil)
			--self:Print("Not Looping Mulgore") -- Debug
		end
	else
		Output:SetScript("OnUpdate", nil)
		--self:Print("Not Looping") -- Debug
	end -- 1.3.0
	--self:Print("'"..tostring(GetRealZoneText()).."' - '"..tostring(GetSubZoneText()).."'") -- Debug line to check Zone info
	if (
		-- Alliance
		(GetRealZoneText() == BL["Elwynn Forest"] and GetSubZoneText() == BL["Goldshire"]) or
		(GetRealZoneText() == BL["Elwynn Forest"] and GetSubZoneText() == BL["Lion's Pride Inn"]) or
		(GetRealZoneText() == BL["Lion's Pride Inn"] and GetSubZoneText() == "")
		) or
	   (
		-- Horde
		--(GetRealZoneText() == BL["Mulgore"] and GetSubZoneText() == "") or -- Big area, sucks big time =(
		(GetRealZoneText() == BL["Mulgore"] and GetSubZoneText() == "" and distance(hx, hy, px, py) <= 3) or -- Trying to reduce the area 1.3.0
		(GetRealZoneText() == BL["Thunder Bluff"] and GetSubZoneText() == BL["Thunder Bluff"]) -- Just so you can buy stuff
		) then

		self:CheckDMF()
		--self:Print("It is: |cFFFF0000"..tostring(isDMF).." |r =)") --!!! Check if the DMF is on !!!
		if isDMF then
			self:LineCheck()
			self:RegisterEvent("BAG_UPDATE", "LineCheck")
			self:RegisterEvent("QUEST_ACCEPTED", "LineCheck")
			self:RegisterEvent("QUEST_LOG_UPDATE", "LineCheck")
			Output:Show()
		end
	else
		if pinWin then
			self:LineCheck()
		else
			self:UnregisterEvent("BAG_UPDATE")
			self:UnregisterEvent("QUEST_ACCEPTED")
			self:UnregisterEvent("QUEST_LOG_UPDATE")
			Output:Hide()
		end
	end
end

--Update Output-window and Buttons
function DQ:LineCheck()
	local line = "|cFF00FF00DMFQuest ("..ver..")|r\n"
	--self:CheckDMF()
	if not isDMF then
		line = line.."|cFFFF0000"..L["Darkmoon Faire"].." "..L["not avaible"].."|r\n"
	end

	if palchemy == nil then palchemy = GetSpellInfo(2259) end -- 1.2.2
	if pinscription == nil then pinscription = GetSpellInfo(45357) end
	if pleatherworking == nil then pleatherworking = GetSpellInfo(2108) end
	if ptailoring == nil then ptailoring = GetSpellInfo(3908) end
	if parchaeology == nil then parchaeology = GetSpellInfo(78670) end
	if pcooking == nil then pcooking = GetSpellInfo(2550) end -- 1.3.0
	if pfirstaid == nil then pfirstaid = GetSpellInfo(3273) end -- 1.3.0
	if pfishing == nil then pfishing = GetSpellInfo(7620) end -- 1.3.0	

	if mjname == nil then mjname = GetItemInfo(1645) end -- 1.2.2
	if lpname == nil then lpname = GetItemInfo(39354) end
	if sbname == nil then sbname = GetItemInfo(6529) end
	if ctname == nil then ctname = GetItemInfo(2320) end
	if bdname == nil then bdname = GetItemInfo(6260) end
	if rdname == nil then rdname = GetItemInfo(2604) end
	if sfname == nil then sfname = GetItemInfo(30817) end
	if fafname == nil then fafname = GetCurrencyInfo(393) end -- 1.2.2

	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
	prof1 = prof1 or 0
	prof2 = prof2 or 0
	archaeology = archaeology or 0
	fishing = fishing or 0
	cooking = cooking or 0
	firstAid = firstAid or 0
	local pname, _, prank = GetProfessionInfo(prof1) -- Primary profession #1
	local sname, _, srank = GetProfessionInfo(prof2) -- Primary profession #2
	local faname, _, farank = GetProfessionInfo(firstAid) -- First Aid -- 1.3.0
	local cname, _, crank = GetProfessionInfo(cooking) -- Cooking
	local finame, _, firank = GetProfessionInfo(fishing) -- Fishing -- 1.3.0
	local aname, _, arank = GetProfessionInfo(archaeology) -- Archaeology
	
	if pname and prank >= 75 then
		if pname == palchemy then
			if IsQuestFlaggedCompleted(29506) then
				line = line..string.format(L["questdone"], pname).."\n"
			else
				local mjcount = GetItemCount(1645) or 0
				if mjcount >= 5 then
					line = line.."|cFF00FF00"..mjcount.."/5|r "..mjname.."\n"
				else
					line = line.."|cFFFF0000"..mjcount.."/5|r "..mjname.." ("..string.format(L["needmore"], (5-mjcount))..")\n"
				end
			end
		elseif pname == pinscription then
			if IsQuestFlaggedCompleted(29515) then
				line = line..string.format(L["questdone"], pname).."\n"
			else
				local lpcount = GetItemCount(39354) or 0
				if lpcount >= 5 then
					line = line.."|cFF00FF00"..lpcount.."/5|r "..lpname.."\n"
				else
					line = line.."|cFFFF0000"..lpcount.."/5|r "..lpname.." ("..string.format(L["needmore"], (5-lpcount))..")\n"
				end
			end
		elseif pname == pleatherworking then
			if IsQuestFlaggedCompleted(29517) then
				--line = line..pname.." quest done\n" -- Before localizations
				line = line..string.format(L["questdone"], pname).."\n"
			else
				local sbcount = GetItemCount(6529) or 0
				local ctcount = GetItemCount(2320) or 0
				local bdcount = GetItemCount(6260) or 0
				if sbcount >= 10 then
					--line = line.."|cFF00FF00"..sbcount.."/10|r Shiny Bauble\n" -- Before localizations
					line = line.."|cFF00FF00"..sbcount.."/10|r "..sbname.."\n"
				else
					--line = line.."|cFFFF0000"..sbcount.."/10|r Shiny Bauble (need "..(10-sbcount).." more)\n" -- Before localizations
					line = line.."|cFFFF0000"..sbcount.."/10|r "..sbname.." ("..string.format(L["needmore"], (10-sbcount))..")\n"
				end
				if ctcount >= 5 then
					line = line.."|cFF00FF00"..ctcount.."/5|r "..ctname.."\n"
				else
					line = line.."|cFFFF0000"..ctcount.."/5|r "..ctname.." ("..string.format(L["needmore"], (5-ctcount))..")\n"
				end
				if bdcount >= 5 then
					line = line.."|cFF00FF00"..bdcount.."/5|r "..bdname.."\n"
				else
					line = line.."|cFFFF0000"..bdcount.."/5|r "..bdname.." ("..string.format(L["needmore"], (5-bdcount))..")\n"
				end
			end
		elseif pname == ptailoring then
			if IsQuestFlaggedCompleted(29520) then
				line = line..string.format(L["questdone"], pname).."\n"
			else
				local ctcount = GetItemCount(2320) or 0
				local bdcount = GetItemCount(6260) or 0
				local rdcount = GetItemCount(2604) or 0
				if ctcount >= 1 then
					line = line.."|cFF00FF00"..ctcount.."/1|r "..ctname.."\n"
				else
					line = line.."|cFFFF0000"..ctcount.."/1|r "..ctname.." ("..string.format(L["needmore"], (1-ctcount))..")\n"
				end
				if bdcount >= 1 then
					line = line.."|cFF00FF00"..bdcount.."/1|r "..bdname.."\n"
				else
					line = line.."|cFFFF0000"..bdcount.."/1|r "..bdname.." ("..string.format(L["needmore"], (1-bdcount))..")\n"
				end
				if rdcount >= 1 then
					line = line.."|cFF00FF00"..rdcount.."/1|r "..rdname.."\n"
				else
					line = line.."|cFFFF0000"..rdcount.."/1|r "..rdname.." ("..string.format(L["needmore"], (1-rdcount))..")\n"
				end
			end
		else
			if IsQuestFlaggedCompleted(29508) or IsQuestFlaggedCompleted(29510) or IsQuestFlaggedCompleted(29511) or IsQuestFlaggedCompleted(29512) or IsQuestFlaggedCompleted(29514) or IsQuestFlaggedCompleted(29516) or IsQuestFlaggedCompleted(29518) or IsQuestFlaggedCompleted(29519) then
				line = line..string.format(L["questdone"], pname).."\n"
			else
				line = line..string.format(L["nomats"], pname).."\n"
			end
		end
	elseif pname and prank < 75 then
		line = line..string.format(L["toolow"], pname, (75-prank)).."\n"
	else
		--line = line.."1st "..L["Primary Profession not learned yet"].."\n"
		line = line..string.format(L["notlearnedyet"], "1st "..L["Primary Profession"]).."\n"
	end

	if sname and srank >= 75 then
		if sname == palchemy then
			if IsQuestFlaggedCompleted(29506) then
				line = line..string.format(L["questdone"], sname).."\n"
			else
				local mjcount = GetItemCount(1645) or 0
				if mjcount >= 5 then
					line = line.."|cFF00FF00"..mjcount.."/5|r "..mjname.."\n"
				else
					line = line.."|cFFFF0000"..mjcount.."/5|r "..mjname.." ("..string.format(L["needmore"], (5-mjcount))..")\n"
				end
			end
		elseif sname == pinscription then
			if IsQuestFlaggedCompleted(29515) then
				line = line..string.format(L["questdone"], sname).."\n"
			else
				local lpcount = GetItemCount(39354) or 0
				if lpcount >= 5 then
					line = line.."|cFF00FF00"..lpcount.."/5|r "..lpname.."\n"
				else
					line = line.."|cFFFF0000"..lpcount.."/5|r "..lpname.." ("..string.format(L["needmore"], (5-lpcount))..")\n"
				end
			end
		elseif sname == pleatherworking then
			if IsQuestFlaggedCompleted(29517) then
				line = line..string.format(L["questdone"], sname).."\n"
			else
				local sbcount = GetItemCount(6529) or 0
				local ctcount = GetItemCount(2320) or 0
				local bdcount = GetItemCount(6260) or 0
				if sbcount >= 10 then
					line = line.."|cFF00FF00"..sbcount.."/10|r "..sbname.."\n"
				else
					line = line.."|cFFFF0000"..sbcount.."/10|r "..sbname.." ("..string.format(L["needmore"], (10-sbcount))..")\n"
				end
				if ctcount >= 5 then
					line = line.."|cFF00FF00"..ctcount.."/5|r "..ctname.."\n"
				else
					line = line.."|cFFFF0000"..ctcount.."/5|r "..ctname.." ("..string.format(L["needmore"], (5-ctcount))..")\n"
				end
				if bdcount >= 5 then
					line = line.."|cFF00FF00"..bdcount.."/5|r "..bdname.."\n"
				else
					line = line.."|cFFFF0000"..bdcount.."/5|r "..bdname.." ("..string.format(L["needmore"], (5-bdcount))..")\n"
				end
			end
		elseif sname == ptailoring then
			if IsQuestFlaggedCompleted(29520) then
				line = line..string.format(L["questdone"], sname).."\n"
			else
				local ctcount = GetItemCount(2320) or 0
				local bdcount = GetItemCount(6260) or 0
				local rdcount = GetItemCount(2604) or 0
				if ctcount >= 1 then
					line = line.."|cFF00FF00"..ctcount.."/1|r "..ctname.."\n"
				else
					line = line.."|cFFFF0000"..ctcount.."/1|r "..ctname.." ("..string.format(L["needmore"], (1-ctcount))..")\n"
				end
				if bdcount >= 1 then
					line = line.."|cFF00FF00"..bdcount.."/1|r "..bdname.."\n"
				else
					line = line.."|cFFFF0000"..bdcount.."/1|r "..bdname.." ("..string.format(L["needmore"], (1-bdcount))..")\n"
				end
				if rdcount >= 1 then
					line = line.."|cFF00FF00"..rdcount.."/1|r "..rdname.."\n"
				else
					line = line.."|cFFFF0000"..rdcount.."/1|r "..rdname.." ("..string.format(L["needmore"], (1-rdcount))..")\n"
				end
			end
		else
			if IsQuestFlaggedCompleted(29508) or IsQuestFlaggedCompleted(29510) or IsQuestFlaggedCompleted(29511) or IsQuestFlaggedCompleted(29512) or IsQuestFlaggedCompleted(29514) or IsQuestFlaggedCompleted(29516) or IsQuestFlaggedCompleted(29518) or IsQuestFlaggedCompleted(29519) then
				line = line..string.format(L["questdone"], sname).."\n"
			else
				line = line..string.format(L["nomats"], sname).."\n"
			end
		end
	elseif sname and srank < 75 then
		line = line..string.format(L["toolow"], sname, (75-srank)).."\n"
	else
		--line = line.."2nd "..L["Primary Profession not learned yet"].."\n"
		line = line..string.format(L["notlearnedyet"], "2nd "..L["Primary Profession"]).."\n"
	end

	if aname and arank >= 75 then
		if IsQuestFlaggedCompleted(29507) then
				line = line..string.format(L["questdone"], aname).."\n"
		else
--			local fafcount = 0 -- Old stuff
--			local fafname = L["Fossil Archaeology Fragment"]
--			for y = 1, GetNumArchaeologyRaces() do
--				local raceName, raceTexture, raceItemID, numFragmentsCollected, _ = GetArchaeologyRaceInfo(y)
--				--self:Print(y..": "..tostring(raceTexture).." - "..tostring(numFragmentsCollected)) -- Printy Printy
--				if (raceTexture == "Interface\\Archeology\\Arch-Race-Fossil") then
--					fafcount = numFragmentsCollected or 0
--					fafname = raceName or L["Fossil Archaeology Fragment"]
--				end
--			end
			local _, fafcount = GetCurrencyInfo(393)
			fafcount = fafcount or 0 -- 1.3.3
			if fafcount >= 15 then
				line = line.."|cFF00FF00"..fafcount.."/15|r "..fafname.."\n"
			else
				line = line.."|cFFFF0000"..fafcount.."/15|r "..fafname.." ("..string.format(L["needmore"], (15-fafcount))..")\n"
			end
		end
	elseif aname and arank < 75 then
		line = line..string.format(L["toolow"], aname, (75-arank)).."\n"
	else
		--line = line..L["Archaeology not learned yet"].."\n"
		--line = line..string.format(L["notlearnedyet"], L["Archaeology"]).."\n"
		line = line..string.format(L["notlearnedyet"], parchaeology).."\n" -- 1.3.0
	end

	if cname and crank >= 75 then
		if IsQuestFlaggedCompleted(29509) then
				line = line..string.format(L["questdone"], cname).."\n"
		else
			local sfcount = GetItemCount(30817) or 0
			if sfcount >= 5 then
				line = line.."|cFF00FF00"..sfcount.."/5|r "..sfname.."\n"
			else
				line = line.."|cFFFF0000"..sfcount.."/5|r "..sfname.." ("..string.format(L["needmore"], (5-sfcount))..")\n"
			end
		end
	elseif cname and crank < 75 then -- or not crank then
		line = line..string.format(L["toolow"], cname, (75-crank)).."\n"
	else
		--line = line..L["Cooking not learned yet"]
		--line = line..string.format(L["notlearnedyet"], L["Cooking"])
		line = line..string.format(L["notlearnedyet"], pcooking).."\n" -- 1.3.0
	end

	if faname and farank >= 75 then -- 1.3.0
		if IsQuestFlaggedCompleted(29512) then
				line = line..string.format(L["questdone"], faname).."\n"
		else
			line = line..string.format(L["nomats"], faname).."\n"
		end
	elseif faname and farank < 75 then
		line = line..string.format(L["toolow"], faname, (75-farank)).."\n"
	else
		line = line..string.format(L["notlearnedyet"], pfirstaid).."\n"
	end

	if finame and firank >= 75 then -- 1.3.0
		if IsQuestFlaggedCompleted(29513) then
				line = line..string.format(L["questdone"], finame)
		else
			line = line..string.format(L["nomats"], finame)
		end
	elseif finame and firank < 75 then
		line = line..string.format(L["toolow"], finame, (75-firank))
	else
		line = line..string.format(L["notlearnedyet"], pfishing)
	end

	--Fix size of the window
	Output.text:SetText(line)
	local width = math.floor(Output.text:GetStringWidth()+0.5) or 350
	local tbl = { strsplit("\n", line) }
	local wkorj = 20
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Tukui") then
		wkorj = 5 -- Smaller margins on ElvUI and TukUI
	end	
	local kork = tonumber(#tbl)*(height+6)+wkorj
	local lev = width+2*wkorj
	if lev < 225 then -- Don't make window smaller than button row below
		lev = 225
	end
	Output:SetHeight(kork)
	Output:SetWidth(lev)

	--All about Buttons below this point
	local korj = math.floor((lev-225)/2)
	local ykorj = 5
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Tukui") then
		ykorj = 8 -- Place buttons little bit lower on ElvUI and TukUI because of smaller margin of main window
	end

	for i,ibutton in ipairs(Output.Buttons) do -- 1.3.2
		ibutton:SetPoint("TOPLEFT", Output, "BOTTOMLEFT", korj+(i-1)*25, 20-ykorj)
	end

	local i = 1
	for questID, itemID in pairs(taul) do
		Output.Buttons[i].texture = Output.Buttons[i]:CreateTexture()
		Output.Buttons[i].texture:SetAllPoints(Output.Buttons[i])
		Output.Buttons[i].texture:SetTexture(select(-2, GetItemInfo(itemID)))

		if IsQuestFlaggedCompleted(questID) then -- Quest done
			Output.Buttons[i]:SetScript("OnEnter",function() tEnter(L["Quest completed"], string.format(L["questitemdone"], GetItemInfo(itemID))) end)
			Output.Buttons[i]:SetScript("OnClick",function() return end)
			Output.Buttons[i].texture:SetVertexColor(0.4, 1, 0.4, 1)
		elseif fQuest(questID) then -- On quest
			Output.Buttons[i]:SetScript("OnEnter",function() tEnter(L["Item found"], string.format(L["questitem"], GetItemInfo(itemID))) end)
			Output.Buttons[i]:SetScript("OnClick",function() return end)
			Output.Buttons[i].texture:SetVertexColor(0.25, 0.25, 1, 1)
		elseif GetItemCount(itemID) == 0 then -- No item
			Output.Buttons[i]:SetScript("OnEnter",function() tEnter(L["Item not found"], string.format(L["noquestitem"], GetItemInfo(itemID))) end)
			Output.Buttons[i]:SetScript("OnClick",function() return end)
			Output.Buttons[i].texture:SetVertexColor(1, 0.4, 0.4, 1)
		else -- Item, no quest
			Output.Buttons[i]:SetScript("OnEnter",function() tEnter(L["Item found"], string.format(L["questitem"], GetItemInfo(itemID))) end)
			Output.Buttons[i]:SetScript("OnClick",function() UseContainerItem(getSlot(itemID)) end)
			--Output.Buttons[i].texture:SetVertexColor(1, 1, 1, 1)
		end

		if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Tukui") then
			skinIcon(Output.Buttons[i])
		end
		Output.Buttons[i]:SetNormalTexture(Output.Buttons[i].texture)

		i = i + 1
	end

end

function DQ:OnInitialize()
	DEFAULT_CHAT_FRAME:AddMessage("DMFQuest: |cFF00FF00Loaded|r")
	self:RegisterChatCommand("dmf", "SlashCMD")
	self:RegisterChatCommand("dmfq", "SlashCMD")
	self:RegisterChatCommand("dmfquest", "SlashCMD")
end

function DQ:OnEnable()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckDMPZ")
	self:RegisterEvent("ZONE_CHANGED", "CheckDMPZ")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "CheckDMPZ")

	--Skinning
	if IsAddOnLoaded("ElvUI") or IsAddOnLoaded("Tukui") then
		local function skinFrame(frame)
			frame:StripTextures(true)
			frame:SetTemplate("Transparent")
		end
		if IsAddOnLoaded("ElvUI") then
			local E, L, V, P, G, DF = unpack(ElvUI)
			local S = E:GetModule('Skins')

			skinFrame(DMFQOutput)
			for _,ibutton in ipairs(Output.Buttons) do -- 1.3.2
				skinFrame(ibutton)
			end			
		else
			local T, C, L = unpack(Tukui)

			skinFrame(DMFQOutput)
			for _,ibutton in ipairs(Output.Buttons) do -- 1.3.2
				skinFrame(ibutton)
			end
		end
	end

	self:CheckDMF()
end